#pragma once
#include "fileparser.h"
#include "graph.h"
#include <time.h>
#include <QObject>

/*!
* Super class for all Data Objects
* Data objects are responsible for generating a graph from the datasets read from the files specified by the passed filenames
*/
class Data : public QObject {
	Q_OBJECT
public:
	/*!
	* Constructor for a Data object
	* \param g pointer to graph that is to be filled with data
	* \param nPath String denoting the path to the file containing the graph's nodes
	* \param ePath String denoting the path to the file containing the graph's edges
	* \param ePath String denoting the path to the file containing the graph's group //this property is currently unused
	*/
	Data(Graph* g, QString nPath = 0, QString ePath = 0, QString gPath = 0) {
		graph = g; 
		nodePath = nPath;
		edgePath = ePath;
		groupPath = gPath;
		srand (time(NULL));
	};
	/*!
	* Destructor for a Data object
	*/
	virtual ~Data(void) {delete input;};
	/*!
	* initGraph() calls the method for graph generation from the data files
	*/
	virtual void initGraph(){};
protected:
	FileParser*		input;
	Graph*			graph;
	QString			nodePath,
					edgePath,
					groupPath;
	/*! returns a random data point with coordinates between 0.05 and 0.95 \return QPointF with random coordinates.*/
	QPointF			randomPoint() {return QPointF(random(), random());};
	/*! returns a random value between 0.05 and 0.95*/
	double			random() {return (rand()/(double)(RAND_MAX + 1)) * 0.90 + 0.05;};
signals:
	/*! signal to notify the main window's status bar of the progress of graph generation */
	void setStatus(QString s);
};

