#pragma once
#include "qstring.h"

/** Struct for Airport Objects
|%AirportID|Name	  |City	       |Country		     |IATA	    |ICAO  |Latitude |Longitude	|Altitude|Timezone|	DST|
|----------|----------|------------|-----------------|----------|------|---------|----------|--------|--------|----|
|507	   |"heathrow"|	"london"   |"united kingdom" |"lhr"	    |"egll"|51.4775	 |-0.461389	|83		 |0		  |"e" |
|26	       |"kugaaruk"|	"pelly bay"|"canada"		 |"ybb"	    |"cybb"|68.534444|-89.808056|56		 |-6	  |"a" |
|3127	   |"pokhara" |	"pokhara"  |"nepal"			 |"pkr"	    |"vnpk"|28.200881|83.982056	|2712	 |5.75	  |"n" |
|... 	   |...       |	...        |... 			 |... 	    |...   |...      |...    	|... 	 |... 	  |... |
*/
struct Airport {
	int			airport_ID;
	QString		name;
	QString		city;
	QString		country;
	QString		IATA;
	QString		ICAO;
	QString		DST;
	double		latitude;
	double		longitude;
	double		altitude;
	double		timezone;

	Airport(int id, const QString& n, const QString& ci, const QString& ct, const QString& iata, const QString& icao, double lat, double lon, double alt, double time, const QString& dst):
	airport_ID(id), 
	name(n),
	city(ci),
	country(ct),
	IATA(iata),
	ICAO(icao), 
	latitude(lat),
	longitude(lon),
	altitude(alt),
	timezone(time),
	DST(dst) {};
	QString		toString() {
		return	QString::number(airport_ID) + ", " +
				name + ", " + 
				city + ", " + 
				country + ", " + 
				IATA + ", " + 
				ICAO + ", " + 
				QString::number(latitude) + ", " + 
				QString::number(longitude) + ", " + 
				QString::number(altitude) + ", " + 
				QString::number(timezone) + ", " + 
				DST;
	};
};

/** Struct for Route Objects
|Airline|AirlineID|Origin|OriginID|Destination|DestinationID|Codeshare|Stops|Equip|
|-------|---------|------|--------|-----------|-------------|---------|-----|-----|
|BA		|1355	  |SIN	 |3316	  |LHR		  |507			|0		  |744  |777  |
|BA		|1355	  |SIN 	 |3316	  |MEL		  |3339	        |Y		  |0	|744  |
|TOM	|5013	  |ACE	 |1055	  |BFS		  |465			|0		  |320  |777  |
|...	|...	  |...	 |...	  |...		  |...			|...	  |...  |...  |
*/
struct Route {
	QString		airline;
	int			airline_ID;
	QString		origin;
	int			origin_ID;
	QString		destination;
	int			destination_ID;
	QString		codeshare;
	QString		equipment;
	int			stops;

	Route(const QString& a, int a_id, const QString& o, int o_id, const QString& d, int d_id, const QString& c, int s, const QString& e):
	airline(a), 
	airline_ID(a_id), 
	origin(o), 
	origin_ID(o_id), 
	destination(d), 
	destination_ID(d_id), 
	codeshare(c), 
	stops(s), 
	equipment(e) {};
	QString		toString() {
		return	airline + ", " +
				QString::number(airline_ID) + ", " + 
				origin + ", " + 
				QString::number(origin_ID) + ", " + 
				destination + ", " + 
				QString::number(destination_ID) + ", " + 
				codeshare + ", " + 
				QString::number(stops) + ", " + 
				equipment;
	};
};