#define _USE_MATH_DEFINES
#include <math.h>
#include "flightdata.h"
#include <qDebug>

FlightData::~FlightData(void) {
	qDeleteAll(airports.begin(), airports.end());
	airports.clear();

	qDeleteAll(routes.begin(), routes.end());
	routes.clear();
}

void FlightData::initGraph(void) {
	input = new FileParser();

	bool success = true;
	emit setStatus("Reading from file...");
	success &= input->read(nodePath, airports);
	qDebug() << airports.size() << " airports parsed.";

	success &= input->read(edgePath, routes);
	qDebug() << routes.size() << " routes parsed.";

	if(success) 
		initEdges(airports, routes);
	else
		qDebug() << "Something went wrong in graph initialization!";
}

void FlightData::initEdges(QList<Airport*> &airports, QList<Route*> &routes) {
	
	QList<int> airportIDs;
	int numAirports = airports.count();

	for (int iAirport = 0; iAirport < numAirports; ++iAirport) {
		Airport* airport = airports[iAirport];

		//construct a list of all airport IDs in order to be able search for the indices in the next step
		airportIDs << airport->airport_ID;

		//initialize airport
		double longitude = airport->longitude;
		double latitude = airport->latitude;

		////Mercator projection of map points
		//normalized to [0; 1]-range
		double proj_y = log(tan(M_PI_4 * (1.0 + latitude / 90.0))); 
		double y = (1.0 - proj_y / M_PI) / 2.0;
		y = qMax(0.0, qMin(y, 1.0)); //projection is undefined for extreme values at the poles 

		double x = (1.0 + longitude / 180.0) / 2.0;   

		Node* node = new Node();
		node->set(x, y);
		node->setInfo(airport->name);
		graph->addNode(node);

		emit setStatus("Creating Graph | Calculating nodes: " + QString::number((iAirport * 100) / numAirports) + "%");
	}

	int numRoutes = routes.count();
	//iterate through all routes and find relevant origin and destination airports
	for (int iRoute = 0; iRoute < numRoutes; ++iRoute) {
		Route* route = routes[iRoute];

		int originPos = airportIDs.indexOf(route->origin_ID);
		int destinationPos = airportIDs.indexOf(route->destination_ID);

		if(originPos == -1 || destinationPos == -1) {
			qDebug() << "This should not happen anymore";
			continue;
		}

		Node* origin = graph->getNodes()->at(originPos);
		Node* destination = graph->getNodes()->at(destinationPos);

		origin->increment();
		destination->increment();

		//initialize edges
		graph->addEdge(new Edge(origin, destination));

		emit setStatus("Creating Graph | Calculating edges: " + QString::number((iRoute * 100) / numRoutes) + "%");
	}
}