#pragma once
#include <QVector>
#include <QMap>
#include <QObject>
#include "graph.h"
/*!
* ForceBundler class performs the main bundling algorithm on the graph passed as an argument.
*/
class ForceBundler: public QObject {
	Q_OBJECT
public:
	ForceBundler() {};
	~ForceBundler(void){};
	/*!
	* sets the ForceBundler's graph to a new pointer.
	* \param g pointer to a new graph.
	*/
	void setGraph(Graph* g) {graph = g;};
	/*!
	* bundleEdges performs the main bundling algorithm on the graph.
	* \param cycles determines how many cycles the edge bundling process iterates over.
	* \param iterations specifies the number of iterations within one cycle.
	* \param springConstant denotes the stiffness of the edges
	*/
	void bundleEdges(int cycles, int iterations, double springConstant);
	
	/*! UNUSED:
	* Deprecated approach that also calculates pairwise compatibilities for the list of passed edges. Does not sort out smaller compatibilities.
	* \param edges list of edges the pairwise compatibilities are to be calculated from
	* \return a vector that stores for each edge a vector containing the compatibilities to all other edges
	*/
	QVector<QVector<double>>* calculateCompatibilities(QList<Edge*> *edges);
	/*!
	* for performance reasons, the costly compatibility calculation is performed only once per bundling call and stored in a Map.
	* \param edges list of edges the pairwise compatibilities are to be calculated from
	* \return a vector that stores for each edge a map of compatible edges with a compatibility > 0.1
	*/
	QVector<QMap<int, double>>* compatibleEdges(QList<Edge*> *edges);

protected:
	Graph* graph;
signals:
	/*! TODO
	* signal to notify the main window's progress bar of the progress of graph generation
	*/
	void setProgress(int p);
};

