#define _USE_MATH_DEFINES
#include <math.h>
#include "marveldata.h"
#include <qDebug>

#include <iostream>
#include <fstream>

void MarvelData::initGraph(void) {
	input = new FileParser();

	bool success = true;

	success &= input->read(nodePath, characters);
	qDebug() << characters.size() << " characters parsed.";

	success &= input->read(edgePath, connections);
	qDebug() << connections.size() << " connections parsed.";

	if(success) 
		initEdges(characters, connections);
		//wrangle(characters, connections);
	else
		qDebug() << "Something went wrong in graph initialization!";
}

void MarvelData::initEdges(QMap<QString, int> &characters, QMap<int, int> &connections) {
	
	QList<int> characterIDs;
	QMapIterator<QString, int> charIter(characters);
	 while (charIter.hasNext()) {
		 charIter.next();

		 characterIDs << charIter.value();

		//initialize character node

		Node* node = new Node();
		node->set(randomPoint());
		node->setInfo(charIter.key());
		graph->addNode(node);
	 }
	
	//iterate through all connections and find relevant characters
	 QMapIterator<int, int> connIter(connections);
	 while (connIter.hasNext()) {
		connIter.next();

		int firstPos = characterIDs.indexOf(connIter.key());
		int secondPos = characterIDs.indexOf(connIter.value());

		if(firstPos == -1 || secondPos == -1) {
			qDebug() << "This should not happen anymore";
			continue;
		}

		Node* first  = graph->getNodes()->at(firstPos);
		Node* second = graph->getNodes()->at(secondPos);

		first->increment();
		second->increment();

		//initialize edges
		graph->addEdge(new Edge(first, second));
	}
}

/* this was only used for dataset reduction purposes. IGNORE! */
/*void MarvelData::wrangle(QMap<QString, int> &characters, QMap<int, int> &connections) {
	
	QList<int> characterIDs;

	QMapIterator<QString, int> charIter(characters);
	while (charIter.hasNext()) {
		charIter.next();
		//Character* character = *charIter;

		//construct a list of all character IDs in order to be able search for the indices in the next step
		characterIDs << charIter.value();
	}

	QList<Book*> books;
	for (int i = 0; i < 12942; i++) {
		Book* book = new Book();
		book->ID = i;
		books.append(book);
	}
	//6487
	//19428

	//create books
	QMapIterator<int, int> connIter(connections);
	while (connIter.hasNext()) {
		connIter.next();
		int book = connIter.value() - 6487;
		int character = connIter.key();

		Book* b = books[book];
		if(characterIDs.contains(character))
			b->addCharacter(character);
	}
	qDebug() << books.size() << " books created.";

	QList<CharStruct*> characterConnections;

	for (int i = 0; i < characterIDs.size(); i++) {
		characterConnections.append(new CharStruct(characterIDs[i]));
	}

	for (int i = 0; i < books.size(); i++) {
		Book* book = books[i];
		QList<int> c = book->characters;

		int c1, c2;
		for(int i = 0; i < c.size(); i++) {
			c1 = c[i];
			int charPos = characterIDs.indexOf(c1);
			for(int j = i + 1; j < c.size(); j++) {
				c2 = c[j];
				if(c1 < c2) {
					if(!characterConnections[charPos]->connections.contains(c2))
						characterConnections[charPos]->connections.append(c2);	
				}
			}
		}
	}

	//iterate through all books
	std::ofstream outputFile;
	outputFile.open("Resources/marvel/cleanup_connections.dat");
	int minConnections = 10000, maxConnections = 0;
	for (QList<CharStruct*>::iterator chIter = characterConnections.begin(); chIter != characterConnections.end(); ++chIter) {
		CharStruct* ch = *chIter;
		
		QList<int> conns = ch->connections;
		if(conns.size() < minConnections) minConnections = conns.size();
		if(conns.size() > maxConnections) maxConnections = conns.size();
		if(minConnections == 0) 
			qDebug() << ch->ID;
		for(int i = 0; i < conns.size(); i++) {
			outputFile << ch->ID << " " << conns[i] << endl;
		}
	}
	outputFile.close();
	qDebug() << "Written connection file!\n";
	qDebug() << "Min connections: " << minConnections;
	qDebug() << "Max connections: " << maxConnections;
}*/