#define _USE_MATH_DEFINES
#include <math.h>
#include "miserablesdata.h"
#include <QDebug>

void MiserablesData::initGraph(void) {
	input = new FileParser();

	bool success = true;

	success &= input->read(nodePath, characters);
	qDebug() << characters.size() << " characters parsed.";
	
	success &= input->read(edgePath, connections);
	qDebug() << connections.size() << " connections parsed.";

	success &= input->read(groupPath, groups);
	qDebug() << groups.size() << " groups parsed.";
	
	if(success) 
		initEdges(characters, groups, connections);
	else
		qDebug() << "Something went wrong in graph initialization!";
}

void MiserablesData::initEdges(QMap<QString, int> &characters, QMap<int, int> &groups, QMap<int, int> &connections) {
	QList<int> characterIDs;
	
	QMapIterator<QString, int> charIter(characters);
	while (charIter.hasNext()) {
		charIter.next();

		//construct a list of all character IDs in order to be able search for the indices in the next step
		characterIDs << charIter.value();

		//initialize character node

		Node* node = new Node();
		
		//QPointF randomPos = QPointF(rand()/(double)(RAND_MAX + 1), rand()/(double)(RAND_MAX + 1));
		node->set(randomPoint());
		node->setInfo(charIter.key());
		node->setGroup(groups.value(charIter.value()));
		graph->addNode(node);
	}
	
	//iterate through all connections and find relevant characters
	QMapIterator<int, int> connIter(connections);
	while (connIter.hasNext()) {
		connIter.next();

		int firstPos = characterIDs.indexOf(connIter.key());
		int secondPos = characterIDs.indexOf(connIter.value());

		if(firstPos == -1 || secondPos == -1) {
			qDebug() << "This should not happen anymore";
			continue;
		}

		Node* first  = graph->getNodes()->at(firstPos);
		Node* second = graph->getNodes()->at(secondPos);

		first->increment();
		second->increment();

		//initialize edges
		graph->addEdge(new Edge(first, second));
	}
}