#ifndef VISUALISIERUNG_H
#define VISUALISIERUNG_H
#pragma once
#include "graph.h"
#include "renderwidget.h"
#include "fileparser.h"
#include "forcebundler.h"
#include <QMainWindow>
#include <QGLWidget>
#include <QGridLayout>
#include <QDebug>
#include <QCheckBox>
#include <QComboBox>
#include <QSpinBox>
#include <QStatusBar>
#include <QPushButton>
#include <QProgressBar>

/*!
* Main application window, inherits QMainWindow 
* Contains all user interface elements as well as the Graph calculation and bundling class
*/
class Visualisierung : public QMainWindow {
	Q_OBJECT

public:
	Visualisierung();
	~Visualisierung();
	
protected:
	Graph			graph;
	ForceBundler	forceBundler;

	/* UI components */
	QWidget			*mainWidget;
	RenderWidget	*renderWidget;
	QProgressBar	*progressBar;
	QWidget			*toolBoxWidget;
	QCheckBox		*wrapCheckBox,
					*backgroundCheckBox,
					*nodesCheckBox,
					*descriptionsCheckBox,
					*edgesCheckBox,
					*bundleCheckBox;
	QSpinBox		*cyclesSpinBox,
					*iterationsSpinBox;
	QDoubleSpinBox	*springSpinBox;
	QComboBox		*datasetComboBox;
	QPushButton		*bundleButton;
	/*              */

	bool			hasBackgroundImage;
	bool			wrapPossible;
	
	bool			bundle;
	int				cycles,
					iterations;
	double			springConstant;

	void initInterface(void);
	void initRenderWidget(void);
	void initToolbox(void);
	void updateToolbox(void);

protected slots:
	void useDataSet(QString n);
	void bundleEdges();
	/*! Updates the text displayed in the status bar. This slot is called from many different places in the application. \param s is the new status bar text. */
	void updateStatus(QString s) {statusBar()->showMessage(s);};
	//TODO
	/*! Updates the progress on the progressbar to a new integer value. \param p is the new integer value. */
	void updateProgress(int p) {progressBar->setValue(p);};
	/*! Updates the cycles member variable to a new integer value. \param c is the new integer value. */
	void setCycles(int c) {cycles = c; bundle = false; updateToolbox();};
	/*! Updates the iterations member variable to a new integer value. \param i is the new integer value. */
	void setIterations(int i) {iterations = i; bundle = false; updateToolbox();};
	/*! Updates the spring constant member variable to a new integer value. \param s is the new double value. */
	void setSpringConstant(double s) {springConstant = s; bundle = false; updateToolbox();};
};

#endif // VISUALISIERUNG_H
